;===============================================================================
;
; Description:      Retrieve MP3 (MP2, MPA) basic information
; Parameter(s):     File name
; Requirement(s):   None
; Return Value(s):  On Success - array with data:
;                       0 - MPEG version
;                       1 - Layer
;                       2 - Bitrate
;                       3 - Frequency
;                       4 - Channel Mode
;                       5 - Duration
;                       6 - Frames
;                       7 - CRC protected
;                       8 - Copyrighted
;                       9 - Original
;                   On Failure empty string and sets @ERROR:
;                       1 - Info not found
; Author(s):        YDY (Lazycat) <mpc@nm.ru>
; Version:          1.2.00
; Date:             30.12.2004
; Note(s):          None
;
;===============================================================================
Func _GetMP3Info($file)
	Local $data[10], $offset = 1, $isValid = 0, $isVBR = 0
	Local $aVersion = StringSplit("MPEG 2.5|Undefined|MPEG 2.5|MPEG 1", "|")
	Local $aLayer = StringSplit("Undefined|Layer III|Layer II|Layer I", "|")
	Local $sBitrate = ""
	Local $sFrequency = ""
	Local $aChanMode = StringSplit("Stereo|Joint stereo|Dual channel|Mono", "|")
	Local $aFlags = StringSplit("No|Yes", "|")
	Local $head, $nVer, $nLay

	If _FileReadAtOffsetHEX($file, 1, 3) = "494433" Then ; ID3v2 tag found
		$offset = _HEXToSSInt(_FileReadAtOffsetHEX($file, 7, 4)) + 10 ; End of ID3v2 tag
	EndIf

	For $ic = $offset To 4096 + $offset
		$marker = _FileReadAtOffsetHEX($file, $ic, 2)
		$marker = StringLeft($marker, 3)
		If StringInStr("FFF,FFE", $marker) Then ; Assume that is frame start
			$head = _HexToBin(_FileReadAtOffsetHEX($file, $ic, 4))
			$nVer = _GetRBits($head, 19, 2)
			$nLay = _GetRBits($head, 17, 2)
			If ($nVer <> 1) And ($nLay <> 0) Then
				If _FileReadAtOffsetHEX($file, $ic + 36, 4) = "58696E67" Then $isVBR = 1 ; Is this a right way?..
				$isValid = 1
				ExitLoop
			EndIf
		EndIf
	Next

	If Not $isValid Then
		SetError(1) ; Frame not found (not mp3 data?)
		Return ("")
	EndIf

	Select
		Case $nVer = 3
			$sFrequency = "44100|48000|32000|Undefined"
		Case $nVer = 2
			$sFrequency = "22050|24000|16000|Undefined"
		Case $nVer = 0
			$sFrequency = "11025|12000|8000|Undefined"
	EndSelect
	Local $aFrequency = StringSplit($sFrequency, "|")
	$data[3] = _GetData($aFrequency, _GetRBits($head, 10, 2))

	Local $pad = 0, $bitrate, $framesize, $frames, $length, $fps
	If _GetRBits($head, 9, 1) Then $pad = 1

	If $isVBR Then
		$data[2] = "Variable"
	Else
		Select
			Case $nVer = 3 And $nLay = 3
				$sBitrate = "Free|32|64|96|128|160|192|224|256|288|320|352|384|416|448|Undefined"
			Case $nVer = 3 And $nLay = 2
				$sBitrate = "Free|32|48|56|64|80|96|112|128|160|192|224|256|320|384|Undefined"
			Case $nVer = 3 And $nLay = 1
				$sBitrate = "Free|32|40|48|56|64|80|96|112|128|160|192|224|256|320|Undefined"
			Case $nVer = 2 And $nLay = 3
				$sBitrate = "Free|32|48|56|64|80|96|112|128|144|160|176|192|224|256|Undefined"
			Case ($nVer = 2 And $nLay = 2) Or ($nVer = 2 And $nLay = 1)
				$sBitrate = "Free|8|16|24|32|40|48|56|64|80|96|112|128|144|160|Undefined"
		EndSelect
		Local $aBitrate = StringSplit($sBitrate, "|")
		$data[2] = _GetData($aBitrate, _GetRBits($head, 12, 4))
		$bitrate = 1000 * $data[2]
		If $nLay = 3 Then
			$framesize = (((12 * $bitrate) / $data[3]) + $pad) * 4
			$fps = $data[3] / 384
		Else
			$framesize = ((144 * $bitrate) / $data[3]) + $pad
			$fps = $data[3] / 1152
		EndIf
		$frames = FileGetSize($file) / $framesize
		$length = $frames / $fps
	EndIf

	$data[0] = _GetData($aVersion, $nVer)
	$data[1] = _GetData($aLayer, $nLay)
	$data[4] = _GetData($aChanMode, _GetRBits($head, 6, 2))
	$data[5] = "00:" & StringFormat("%02d:%02d", Int($length / 60), $length - Int($length / 60) * 60)
	$data[6] = Int($frames)
	$data[7] = _GetData($aFlags, Not _GetRBits($head, 16, 1)) ; CRC
	$data[8] = _GetData($aFlags, _GetRBits($head, 3, 1)) ; Private
	$data[9] = _GetData($aFlags, _GetRBits($head, 2, 1)) ; Original
	Return ($data)
EndFunc   ;==>_GetMP3Info

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Support functions ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Func _GetRBits($str, $pos, $size)
	Local $ic, $res = 0, $bStr = StringMid($str, 33 - $pos - $size, $size)
	For $ic = 0 To $size - 1
		If StringMid($bStr, $size - $ic, 1) == "1" Then $res = $res + 2 ^ $ic
	Next
	Return ($res)
EndFunc   ;==>_GetRBits

Func _GetData(ByRef $array, $val)
	If $val > UBound($array) - 1 Then Return ("Undefined")
	Return ($array[$val + 1])
EndFunc   ;==>_GetData

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; Common functions ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Func _HEXToSSInt($sHexStr)
	Local $iCnt, $iLen, $sTempStr = "", $iReturn = 0
	For $iCnt = 1 To StringLen($sHexStr) Step 2
		$sTempStr = $sTempStr & StringTrimLeft(_HexToBin(StringMid($sHexStr, $iCnt, 2)), 1)
	Next
	$iLen = StringLen($sTempStr)
	For $iCnt = 0 To $iLen - 1
		$iReturn = $iReturn + Number(StringMid($sTempStr, $iLen - $iCnt, 1)) * 2 ^ $iCnt
	Next
	Return ($iReturn)
EndFunc   ;==>_HEXToSSInt

Func _HexToBin($str)
	Local $res = "", $i
	While StringLen($str) > 0
		$val = Dec(StringRight($str, 1))
		$str = StringTrimRight($str, 1)
		For $i = 1 To 4
			$res = String(Mod($val, 2)) & $res
			$val = Int($val / 2)
		Next
	WEnd
	Return ($res)
EndFunc   ;==>_HexToBin

Func _FileReadAtOffsetHEX($file, $offset, $bytes)
	Local $tfile = FileOpen($file, 0)
	Local $tstr = "", $i
	FileRead($tfile, $offset - 1)
	For $i = $offset To $offset + $bytes - 1
		$tstr = $tstr & Hex(Asc(FileRead($tfile, 1)), 2)
	Next
	FileClose($tfile)
	Return ($tstr)
EndFunc   ;==>_FileReadAtOffsetHEX
